import re
import unittest2
from test.helpers.httpretty_extension import httpretty
import json
import datetime
from quandl.model.dataset import Dataset
from quandl.model.database import Database
from mock import patch, call
from test.factories.dataset import DatasetFactory
from test.factories.meta import MetaFactory


class GetDatasetTest(unittest2.TestCase):

    @classmethod
    def setUpClass(cls):
        httpretty.enable()
        dataset = {'dataset': DatasetFactory.build(
            database_code='NSE', dataset_code='OIL')}
        httpretty.register_uri(httpretty.GET,
                               re.compile(
                                   'https://www.quandl.com/api/v3/datasets/*'),
                               body=json.dumps(dataset))
        dataset_code = Dataset.get_code_from_meta(dataset['dataset'])
        cls.dataset_instance = Dataset(dataset_code, dataset['dataset'])

    @classmethod
    def tearDownClass(cls):
        httpretty.disable()
        httpretty.reset()

    @patch('quandl.connection.Connection.request')
    def test_dataset_calls_connection(self, mock):
        d = Dataset('NSE/OIL')
        d.data_fields()
        expected = call('get', 'datasets/NSE/OIL/metadata', params={})
        self.assertEqual(mock.call_args, expected)

    def test_dataset_returns_dataset_object(self):
        dataset = Dataset('NSE/OIL')
        self.assertIsInstance(dataset, Dataset)
        self.assertEqual(dataset.dataset_code, 'OIL')

    def test_dataset_attributes_are_datetime_objects(self):
        dataset = Dataset('NSE/OIL')
        self.assertIsInstance(dataset.refreshed_at, datetime.datetime)
        self.assertIsInstance(dataset.newest_available_date, datetime.date)

    def test_dataset_column_names_match_expected(self):
        dataset = Dataset('NSE/OIL')
        self.assertItemsEqual(
            dataset.column_names, ['Date', 'column.1', 'column.2', 'column.3'])

    def test_dataset_database_gives_instance_of_database(self):
        database = self.dataset_instance.database()
        self.assertIsInstance(database, Database)

    @patch('quandl.model.data.Data.all')
    def test_dataset_data_calls_data_all(self, mock):
        self.dataset_instance.data(
            params={'start_date': '2015-07-01', 'end_date': '2015-07-10'})
        expected = call(params={'dataset_code': 'OIL', 'database_code': 'NSE',
                                'start_date': '2015-07-01', 'end_date': '2015-07-10',
                                'order': 'asc'})
        self.assertEqual(mock.call_args, expected)


class ListDatasetsTest(unittest2.TestCase):

    @classmethod
    def setUpClass(cls):
        httpretty.enable()
        datasets = {'datasets': DatasetFactory.build_batch(3)}
        meta = {'meta': MetaFactory.build(current_page=1, total_pages=1)}
        datasets.update(meta)
        httpretty.register_uri(httpretty.GET,
                               re.compile(
                                   'https://www.quandl.com/api/v3/datasets*'),
                               body=json.dumps(datasets))

    @classmethod
    def tearDownClass(cls):
        httpretty.disable()
        httpretty.reset()

    @patch('quandl.connection.Connection.request')
    def test_datasets_calls_connection(self, mock):
        Dataset.all()
        expected = call('get', 'datasets', params={})
        self.assertEqual(mock.call_args, expected)

    def test_datasets_return_dataset_objects(self):
        results = Dataset.all()
        self.assertEqual(len(results), 3)
        for result in results:
            self.assertIsInstance(result, Dataset)

    def test_datasets_has_more_results(self):
        results = Dataset.all()
        self.assertFalse(results.has_more_results())

    def test_values_exist(self):
        results = Dataset.all()
        self.assertIsInstance(results.values, list)
        self.assertIsInstance(results.values[0], Dataset)

    def test_meta_attributes_can_be_accessed(self):
        results = Dataset.all()
        self.assertEqual(results.meta['current_page'], 1)
        self.assertEqual(results.current_page, 1)

    def test_to_list_returns_list(self):
        results = Dataset.all()
        data = results.to_list()
        self.assertIsInstance(data, list)
        self.assertIsInstance(data[0], list)



from quandl.operations.get import GetOperation
from quandl.operations.list import ListOperation
from quandl.util import Util
from .model_base import ModelBase
from .data import Data
from .data_list import DataList
import quandl.model.database
import six
from quandl.errors.quandl_error import (NotFoundError, ColumnNotFound)
from quandl.message import Message


class Dataset(GetOperation, ListOperation, ModelBase):

    @classmethod
    def get_path(cls):
        return "%s/metadata" % cls.default_path()

    @classmethod
    def get_code_from_meta(cls, metadata):
        return "%s/%s" % (metadata['database_code'], metadata['dataset_code'])

    def __init__(self, code, raw_data=None, **options):
        ModelBase.__init__(self, code, raw_data)

        parsed_code = self.code.split("/")
        if len(parsed_code) < 2:
            raise SyntaxError(Message.ERROR_INVALID_DATABASE_CODE_FROMAT)

        self.database_code = parsed_code[0]
        self.dataset_code = parsed_code[1]
        self.options = options

    def data(self, **options):
        # handle_not_found_error if set to True will add an empty DataFrame
        # for a non-existent dataset instead of raising an error
        handle_not_found_error = options.pop('handle_not_found_error', False)
        handle_column_not_found = options.pop('handle_column_not_found', False)
        # default order to ascending, and respect whatever user passes in
        params = {
            'database_code': self.database_code,
            'dataset_code': self.dataset_code,
            'order': 'asc'
        }
        updated_options = Util.merge_options('params', params, **options)
        try:
            return Data.all(**updated_options)
        except NotFoundError:
            if handle_not_found_error:
                return DataList(Data, [], {'column_names': [six.u('None'), six.u('Not Found')]})
            raise
        except ColumnNotFound:
            if handle_column_not_found:
                return DataList(Data, [], {'column_names': [six.u('None'), six.u('Not Found')]})
            raise

    def database(self):
        return quandl.model.database.Database(self.database_code)
